from pydantic import BaseModel
from typing import Optional, List
from datetime import datetime


class ProductBase(BaseModel):
    shop_id: int
    category_id: Optional[int] = None
    name: str
    description: Optional[str] = None
    price: float
    stock: int


class ProductCreate(ProductBase):
    pass


class ProductRead(ProductBase):
    id: int
    created_at: datetime
    images: List["ProductImageRead"] = []

    class Config:
        orm_mode = True


class ProductUpdate(BaseModel):
    shop_id: Optional[int] = None
    category_id: Optional[int] = None
    name: Optional[str] = None
    description: Optional[str] = None
    price: Optional[float] = None
    stock: Optional[int] = None


class ProductImageBase(BaseModel):
    product_id: int
    image_url: str


class ProductImageCreate(ProductImageBase):
    pass


class ProductImageRead(ProductImageBase):
    id: int

    class Config:
        orm_mode = True
