import customtkinter as ctk
import os
from tkinter import messagebox
from PIL import Image
from utils.api_requests import login_api

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"


def login_frame(parent, switch_func, API_URL):
    # Overall container with dark background
    container = ctk.CTkFrame(parent, fg_color=DARK_BG)
    container.grid_columnconfigure(0, weight=1)
    container.grid_columnconfigure(1, weight=1)
    container.grid_rowconfigure(0, weight=1)

    # Left image frame
    left_frame = ctk.CTkFrame(container, fg_color=CARD_BG, corner_radius=15)
    left_frame.grid(row=0, column=0, sticky="nsew", padx=(20, 10), pady=20)

    # App branding
    brand_frame = ctk.CTkFrame(left_frame, fg_color="transparent")
    brand_frame.place(relx=0.5, rely=0.5, anchor="center")

    app_name = ctk.CTkLabel(
        brand_frame,
        text="Shopping App",
        font=("Helvetica", 40, "bold"),
        text_color=SHOPPING,
    )
    app_name.pack()

    app_slogan = ctk.CTkLabel(
        brand_frame,
        text="Your One-Stop Shopping Destination",
        font=("Helvetica", 16),
        text_color="white",
    )
    app_slogan.pack(pady=(10, 30))

    # Right login frame
    right_frame = ctk.CTkFrame(container, fg_color=CARD_BG, corner_radius=15)
    right_frame.grid(row=0, column=1, sticky="nsew", padx=(10, 20), pady=20)
    right_frame.grid_columnconfigure(0, weight=1)

    # Login Function
    def login():
        email = entry_email.get()
        password = entry_password.get()

        if not email or not password:
            messagebox.showwarning("Input Error", "Both fields are required!")
            return

        status_code, response_data = login_api(email, password, API_URL)

        if status_code == 200:
            access_token = response_data.get("access_token")
            messagebox.showinfo("Login Successful", f"Welcome back, {email}!")
            switch_func("dashboard", access_token)
        else:
            messagebox.showerror(
                "Login Failed", response_data.get("detail", "Invalid credentials")
            )

    # Login form container
    form_frame = ctk.CTkFrame(right_frame, fg_color="transparent")
    form_frame.place(relx=0.5, rely=0.5, anchor="center")

    # Title
    ctk.CTkLabel(
        form_frame,
        text="Welcome Back",
        font=("Helvetica", 32, "bold"),
        text_color=SHOPPING,
    ).pack(pady=(0, 10))

    ctk.CTkLabel(
        form_frame,
        text="Sign in to your account",
        font=("Helvetica", 14),
        text_color="gray",
    ).pack(pady=(0, 20))

    # Email
    email_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
    email_frame.pack(fill="x", pady=(0, 15))

    ctk.CTkLabel(
        email_frame, text="Email", font=("Helvetica", 14), text_color="white"
    ).pack(anchor="w")

    entry_email = ctk.CTkEntry(
        email_frame,
        height=40,
        corner_radius=8,
        placeholder_text="Enter your email",
        fg_color="#3b3b3b",
        border_color=SHOPPING,
        text_color="white",
    )
    entry_email.pack(fill="x", pady=(5, 0))

    # Password
    password_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
    password_frame.pack(fill="x", pady=(0, 20))

    ctk.CTkLabel(
        password_frame, text="Password", font=("Helvetica", 14), text_color="white"
    ).pack(anchor="w")

    entry_password = ctk.CTkEntry(
        password_frame,
        height=40,
        show="*",
        corner_radius=8,
        placeholder_text="Enter your password",
        fg_color="#3b3b3b",
        border_color=SHOPPING,
        text_color="white",
    )
    entry_password.pack(fill="x", pady=(5, 0))

    # Login Button
    ctk.CTkButton(
        form_frame,
        text="Sign In",
        command=login,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    ).pack(fill="x", pady=(0, 15))

    # Register Redirect
    register_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
    register_frame.pack(fill="x")

    ctk.CTkLabel(
        register_frame,
        text="Don't have an account?",
        font=("Helvetica", 12),
        text_color="gray",
    ).pack(side="left", padx=(0, 5))

    ctk.CTkButton(
        register_frame,
        text="Register",
        command=lambda: switch_func("register"),
        fg_color="transparent",
        hover_color="#3b3b3b",
        text_color=SHOPPING,
        font=("Helvetica", 12, "bold"),
        width=70,
        height=25,
    ).pack(side="left")

    return container
