import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox
import io
# import time
# import threading

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
BACKEND_HOST = "http://127.0.0.1:8000"


def fix_url(url):
    """
    If the provided URL does not start with http, assume it's a relative path.
    Remove any unwanted prefix (e.g., "app/static/") and prepend the public URL.
    """
    if url.startswith("http"):
        return url
    # If the URL starts with "app/static/", remove that part.
    prefix = "app/static/"
    if url.startswith(prefix):
        url = url[len(prefix) :]
    # Prepend the public URL
    return f"{BACKEND_HOST}/static/{url}"


def dashboard_frame(parent, switch_func, API_URL, token):
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    # ------------- LEFT SIDEBAR -------------
    sidebar = ctk.CTkFrame(frame, fg_color=CARD_BG, width=60)
    sidebar.pack(side="left", fill="y")
    sidebar.pack_propagate(False)

    # Dashboard Icon
    dashboard_icon = ctk.CTkButton(
        sidebar,
        text="🏠",
        font=("Helvetica", 20),
        fg_color="transparent",
        text_color="white",
        hover_color="#3b3b3b",
        width=40,
        height=40,
        command=lambda: switch_func("dashboard"),
    )
    dashboard_icon.pack(pady=(20, 10))

    # User Icon
    user_icon = ctk.CTkButton(
        sidebar,
        text="👤",
        font=("Helvetica", 20),
        fg_color="transparent",
        text_color="white",
        hover_color="#3b3b3b",
        width=40,
        height=40,
        command=lambda: switch_func("user_details"),
    )
    user_icon.pack(pady=10)

    # Cart Icon
    cart_icon = ctk.CTkButton(
        sidebar,
        text="🛒",
        font=("Helvetica", 20),
        fg_color="transparent",
        text_color="white",
        hover_color="#3b3b3b",
        width=40,
        height=40,
        command=lambda: switch_func("user_orders"),
    )
    cart_icon.pack(pady=10)

    # ------------- MAIN CONTENT -------------
    main_content = ctk.CTkFrame(frame, fg_color="transparent")
    main_content.pack(side="left", fill="both", expand=True)

    # Search bar at top
    search_frame = ctk.CTkFrame(main_content, fg_color=CARD_BG, height=60)
    search_frame.pack(fill="x", padx=20, pady=10)
    search_frame.pack_propagate(False)

    search_entry = ctk.CTkEntry(
        search_frame,
        placeholder_text="Search for shops and products...",
        height=35,
        fg_color="#3b3b3b",
        text_color="white",
        placeholder_text_color="#888888",
    )
    search_entry.pack(side="left", fill="x", expand=True, padx=(20, 10), pady=12)

    def perform_search():
        """Call an endpoint to search products by keyword."""
        keyword = search_entry.get().strip()
        if not keyword:
            messagebox.showinfo("Info", "Please enter a search keyword.")
            return
        try:
            headers = {"Authorization": f"Bearer {token}"}
            # Adjust endpoint/params as needed
            resp = requests.get(
                f"{API_URL}/search?name={keyword}&search_type=products", headers=headers
            )
            if resp.status_code == 200:
                products = resp.json()
                display_products(products, products_frame)
            else:
                messagebox.showerror(
                    "Error", f"Search failed. Status code: {resp.status_code}"
                )
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    search_button = ctk.CTkButton(
        search_frame,
        text="Search",
        fg_color=SHOPPING,
        text_color="white",
        width=100,
        height=35,
        command=perform_search,
    )
    search_button.pack(side="right", padx=(0, 20))

    # Add refresh button with improved design
    def refresh_dashboard():
        """Refresh the dashboard to its initial state"""
        # Clear search entry
        search_entry.delete(0, "end")
        # Reload featured shops and recommendations
        fetch_featured_shops()
        fetch_recommendations()

    refresh_button = ctk.CTkButton(
        search_frame,
        text="⟳",  # Unicode refresh symbol
        fg_color="#3b3b3b",  # Dark background
        text_color="white",
        width=35,
        height=35,
        corner_radius=20,  # Make it circular
        hover_color=SHOPPING,  # Use shopping color on hover
        font=("Helvetica", 20),  # Larger font for the icon
        command=refresh_dashboard,
    )
    refresh_button.pack(side="right", padx=(0, 10))

    # Featured Shops Carousel
    carousel_frame = ctk.CTkFrame(main_content, fg_color=CARD_BG, height=300)
    carousel_frame.pack(fill="x", padx=20, pady=10)
    carousel_frame.pack_propagate(False)

    # Carousel content
    carousel_content = ctk.CTkFrame(carousel_frame, fg_color="transparent")
    carousel_content.pack(fill="both", expand=True, padx=20, pady=20)

    # Navigation buttons frame
    nav_buttons_frame = ctk.CTkFrame(carousel_frame, fg_color="transparent")
    nav_buttons_frame.place(relx=0.98, rely=0.5, anchor="e")

    def prev_slide():
        nonlocal current_shop_index
        current_shop_index = (current_shop_index - 1) % len(featured_shops)
        update_carousel(skip_timer=True)

    def next_slide():
        nonlocal current_shop_index
        current_shop_index = (current_shop_index + 1) % len(featured_shops)
        update_carousel(skip_timer=True)

    # Button styling configuration
    button_config = {
        "width": 35,
        "height": 35,
        "corner_radius": 20,
        "font": ("Helvetica", 16, "bold"),
        "fg_color": "#1f1f1f",  # Dark background color
        "hover_color": SHOPPING,
        "text_color": SHOPPING,  # Text color matches SHOPPING
        "border_width": 2,
        "border_color": SHOPPING,
    }

    prev_button = ctk.CTkButton(
        nav_buttons_frame, text="←", command=prev_slide, **button_config
    )
    prev_button.pack(side="left", padx=3)

    next_button = ctk.CTkButton(
        nav_buttons_frame, text="→", command=next_slide, **button_config
    )
    next_button.pack(side="left", padx=3)

    featured_shops = []
    current_shop_index = 0

    def create_shop_slide(shop_data):
        slide = ctk.CTkFrame(carousel_content, fg_color="transparent")

        # Shop Image
        image_frame = ctk.CTkFrame(slide, fg_color="transparent", width=400, height=200)
        image_frame.pack(side="left", padx=20)
        image_frame.pack_propagate(False)

        image_label = ctk.CTkLabel(image_frame, text="")
        image_label.pack(expand=True)

        image_url = shop_data.get("image_url")
        if image_url:
            fixed_url = fix_url(image_url)
            try:
                resp = requests.get(fixed_url)
                if resp.status_code == 200:
                    pil_img = Image.open(io.BytesIO(resp.content)).resize((380, 180))
                    tk_img = ImageTk.PhotoImage(pil_img)
                    image_label.configure(image=tk_img, text="")
                    image_label.image = tk_img
            except Exception as e:
                print(f"[DEBUG] Shop image error: {e}")

        # Shop Details
        details_frame = ctk.CTkFrame(slide, fg_color="transparent")
        details_frame.pack(side="left", fill="both", expand=True, padx=20)

        shop_name = ctk.CTkLabel(
            details_frame,
            text=shop_data.get("name", "Shop Name"),
            font=("Helvetica", 24, "bold"),
            text_color=SHOPPING,
        )
        shop_name.pack(anchor="w", pady=(0, 10))

        shop_desc = ctk.CTkLabel(
            details_frame,
            text=shop_data.get("description", "No description available."),
            font=("Helvetica", 14),
            text_color="white",
            wraplength=400,
            justify="left",
        )
        shop_desc.pack(anchor="w", pady=5)

        visit_button = ctk.CTkButton(
            details_frame,
            text="Visit Shop →",
            fg_color=SHOPPING,
            text_color="white",
            command=lambda: switch_func("view_shop", shop_data.get("id")),
            width=120,
            height=35,
        )
        visit_button.pack(anchor="w", pady=20)

        return slide

    def update_carousel(skip_timer=False):
        nonlocal current_shop_index
        if not featured_shops:
            return

        # Clear current content
        for widget in carousel_content.winfo_children():
            widget.destroy()

        # Create and show current slide
        current_slide = create_shop_slide(featured_shops[current_shop_index])
        current_slide.pack(fill="both", expand=True)

        # Update dots
        update_carousel_dots()

        # Schedule next slide only if not skipping timer
        if not skip_timer:
            current_shop_index = (current_shop_index + 1) % len(featured_shops)
            carousel_content.after(5000, lambda: update_carousel(False))

    # Carousel navigation dots
    dots_frame = ctk.CTkFrame(carousel_frame, fg_color="transparent", height=30)
    dots_frame.pack(side="bottom", fill="x", pady=10)

    def update_carousel_dots():
        for widget in dots_frame.winfo_children():
            widget.destroy()

        for i in range(len(featured_shops)):
            color = SHOPPING if i == current_shop_index else "gray"
            dot = ctk.CTkLabel(
                dots_frame, text="●", text_color=color, font=("Helvetica", 16)
            )
            dot.pack(side="left", padx=2)

    # Products Section
    products_section = ctk.CTkFrame(main_content, fg_color="transparent")
    products_section.pack(fill="both", expand=True, padx=20, pady=10)

    products_header = ctk.CTkFrame(products_section, fg_color="transparent")
    products_header.pack(fill="x", pady=(0, 10))

    products_title = ctk.CTkLabel(
        products_header,
        text="Featured Products",
        font=("Helvetica", 20, "bold"),
        text_color="white",
    )
    products_title.pack(side="left")

    products_frame = ctk.CTkScrollableFrame(
        products_section, fg_color="transparent", height=400
    )
    products_frame.pack(fill="both", expand=True)

    def create_product_card(parent, product_data):
        card = ctk.CTkFrame(
            parent, fg_color=CARD_BG, corner_radius=10, width=200, height=280
        )
        card.pack_propagate(False)

        # Product Image
        image_frame = ctk.CTkFrame(card, fg_color="transparent", height=150)
        image_frame.pack(fill="x", padx=10, pady=10)
        image_frame.pack_propagate(False)

        image_label = ctk.CTkLabel(image_frame, text="")
        image_label.pack(expand=True)

        product_images = product_data.get("images", [])
        if product_images:
            img_url = product_images[0].get("image_url")
            if img_url:
                fixed_url = fix_url(img_url)
                try:
                    resp = requests.get(fixed_url)
                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize(
                            (120, 120)
                        )
                        tk_img = ImageTk.PhotoImage(pil_img)
                        image_label.configure(image=tk_img, text="")
                        image_label.image = tk_img
                except Exception as e:
                    print(f"[DEBUG] Product image error: {e}")

        # Product Details
        details_frame = ctk.CTkFrame(card, fg_color="transparent")
        details_frame.pack(fill="both", expand=True, padx=10, pady=(0, 10))

        name_label = ctk.CTkLabel(
            details_frame,
            text=product_data.get("name", "No Name"),
            font=("Helvetica", 14, "bold"),
            wraplength=180,
            justify="left",
        )
        name_label.pack(anchor="w")

        price_label = ctk.CTkLabel(
            details_frame,
            text=f"₫ {product_data.get('price', 0.0):,.1f}",
            font=("Helvetica", 14, "bold"),
            text_color="#ff4242",
        )
        price_label.pack(anchor="w", pady=5)

        view_button = ctk.CTkButton(
            details_frame,
            text="View Details",
            command=lambda: switch_func("view_product", product_data),
            fg_color=SHOPPING,
            text_color="white",
            height=30,
        )
        view_button.pack(side="bottom", pady=5)

        return card

    def display_products(products, container):
        for widget in container.winfo_children():
            widget.destroy()

        if not products:
            no_products_label = ctk.CTkLabel(
                container,
                text="No products available.",
                font=("Helvetica", 14),
                text_color="gray",
            )
            no_products_label.pack(pady=20)
            return

        grid_frame = ctk.CTkFrame(container, fg_color="transparent")
        grid_frame.pack(fill="both", expand=True)

        max_cols = 4
        row = 0
        col = 0

        for product in products:
            product_card = create_product_card(grid_frame, product)
            product_card.grid(row=row, column=col, padx=10, pady=10, sticky="nsew")

            col += 1
            if col >= max_cols:
                col = 0
                row += 1

        for i in range(max_cols):
            grid_frame.grid_columnconfigure(i, weight=1)

    def fetch_featured_shops():
        headers = {"Authorization": f"Bearer {token}"}
        try:
            resp = requests.get(f"{API_URL}/shops/list", headers=headers)
            if resp.status_code == 200:
                nonlocal featured_shops
                featured_shops = resp.json()[:5]  # Get top 5 shops for carousel
                update_carousel()
            else:
                messagebox.showerror(
                    "Error",
                    f"Failed to fetch featured shops. Status: {resp.status_code}",
                )
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    def fetch_recommendations():
        headers = {"Authorization": f"Bearer {token}"}
        try:
            resp = requests.get(f"{API_URL}/product/list", headers=headers)
            if resp.status_code == 200:
                products = resp.json()
                display_products(products[:12], products_frame)  # Show top 12 products
            else:
                messagebox.showerror(
                    "Error",
                    f"Failed to fetch recommended products. Status: {resp.status_code}",
                )
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    # Load data on start
    fetch_featured_shops()
    fetch_recommendations()

    return frame
